// Array-of-edges representation
typedef struct GraphRep {
	int   nV;    // #vertices
	int   nE;    // #edges
	int   n;     // size of edge array
	Edge *edges; // array of edges
} GraphRep;

// Usage: vs = neighbours(g, x, &n)
// Pre:  valid(g) and valid(x) && isdefined(n)
// Post: vs[0..n-1] contains neighbours of x


Vertex *neighbours(Graph g, Vertex x, int *nv)
{
	assert(validG(g) && validV(g,x) && nv != NULL);
	int nneighbours = 0;
	int i;
	for (i = 0; i < g->nE; i++) {
		if (g->edges[i].v == x || g->edges[i].w == x)
			nneighbours++;
	}
	Vertex *ns = malloc(nneighbours*sizeof(Vertex));
	int n = 0;
	for (i = 0; i < g->nE; i++) {
		if (g->edges[i].v == x)
			ns[n++] = g->edges[i].w;
		else if (g->edges[i].w == x)
			ns[n++] = g->edges[i].v;
	}
	*nv = n;
	return ns;
}

// faster but less space efficient
Vertex *neighbours(Graph g, Vertex x, int *nv)
{
	assert(validG(g) && validV(g,x) && nv != NULL);
	Vertex *ns = malloc((g->nV-1)*sizeof(Vertex));
	int n = 0;
	for (i = 0; i < g->nE; i++) {
		if (g->edges[i].v == x)
			ns[n++] = g->edges[i].w;
		else if (g->edges[i].w == x)
			ns[n++] = g->edges[i].v;
	}
	*nv = n;
	return ns;
}
